package com.kexun.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.kexun.common.utils.Result;
import com.kexun.entity.BusInfoModifyHis;
import com.kexun.entity.FieldDefineEntity;
import com.kexun.hander.PageBean;
import com.kexun.model.ro.FiledRO;
import com.kexun.model.ro.FiledUpdateRO;
import com.kexun.service.FieldDefineService;
import com.kexun.utils.PageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 字段定义表 前端控制器
 * </p>
 *
 * @author lgs
 * @since 2023-06-21
 */
@RestController
@RequestMapping("/admin/")
public class FieldDefineController {

    @Autowired
    FieldDefineService fieldDefineService;

    /**
     * 字段列表查询
     *
     * @return
     */
    @GetMapping("filed-list")
    public Result list(@RequestBody FiledRO filedRO) {
        Result result = new Result();
        PageBean<BusInfoModifyHis> page = new PageBean<>(PageUtils.getPageNo(filedRO.getPageNo()), PageUtils.getPageSize(filedRO.getPageSize()));
        if (StringUtils.isEmpty(filedRO.getCategory())) {
            return Result.error("-1", "字段类别不能为空");
        }
        IPage<FieldDefineEntity> fieldDefineEntityIPage = fieldDefineService.listPage(filedRO);
        result.setData(fieldDefineEntityIPage);
        return result;
    }

    /**
     * 字段列表详情
     *
     * @param id :字段列表主键
     * @return
     */
    @GetMapping("filed-detail")
    public Result detail(@RequestParam Long id) {
        Result result = new Result();
        if (StringUtils.isEmpty(id)) {
            return Result.error("-1", "字段定义的ID不能为空");
        }
        FieldDefineEntity fieldDefineEntity = fieldDefineService.getById(id);
        result.setData(fieldDefineEntity);
        return result;
    }

    /**
     * 保存或者修改字段定义信息
     * @param filedUpdateRO
     * @return
     */
    @PostMapping("filed-update")
    public Result saveOrUpdate(@RequestBody FiledUpdateRO filedUpdateRO){
        fieldDefineService.saveOrUpdate(filedUpdateRO);
        return Result.success();
    }

    /**
     * 基础字段删除
     * @param id 主键ID
     * @return
     */
    @DeleteMapping("filed-delete")
    public Result delete(@RequestParam Long id){
        if (StringUtils.isEmpty(id)) {
            return Result.error("-1", "字段定义的ID不能为空");
        }
        fieldDefineService.removeById(id);
        return Result.success();
    }

    /**
     * 基础字段启用/停用
     * @param id 主键
     * @param status 开启关闭状态
     * @return
     */
    @PostMapping("filed-onoff")
    public Result onOff(@RequestParam Long id,@RequestParam String status){
        if (StringUtils.isEmpty(id)) {
            return Result.error("-1", "字段定义的ID不能为空");
        }
        if (StringUtils.isEmpty(status)) {
            return Result.error("-1", "开启/停用状态不能为空");
        }
        fieldDefineService.updateOnOff(id,status);
        return Result.success();
    }


}
