package com.kexun.controller;


import com.kexun.common.utils.Result;
import com.kexun.model.ro.ScriptRequest;
import com.kexun.model.vo.TestVo;
import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import groovy.lang.Script;
import org.codehaus.groovy.control.CompilerConfiguration;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Hashtable;
import java.util.Map;

/**
 * <p>
 * 测试类
 * </p>
 *
 * @author lgs
 * @since 2023-05-31
 */
@RestController
@RequestMapping("/admin")
public class TestController {

    private static final Object lock = new Object();
    private static final GroovyShell groovyShell;
    private static Hashtable<String, Script> scriptCache = new Hashtable<>();

    /**
     * 默认绑定已有方法的实例
     */
    @Autowired
    private Binding groovyBinding;

    static {
        CompilerConfiguration cfg = new CompilerConfiguration();
        groovyShell = new GroovyShell(cfg);
    }

    /**
     * 测试
     * @param name
     * @return
     */
    @GetMapping("list")
    public Result list(String name) {
        TestVo testVo = new TestVo();
        testVo.setId(12345678901021L);
        testVo.setName("中国");
        return Result.success("ok", testVo);
    }

    @PostMapping(value = "/execute")
    public void ruleExecutor(@RequestBody ScriptRequest request) {
        if (request.getParamMap() == null) {
            Object o = ruleParse(request.getExpression());
            System.out.println(o);
        } else {
            Object o = ruleParse(request.getExpression(), request.getParamMap());
            System.out.println(o);
        }
    }

    /**
     * 在客户端本地只实例化单例RuleExecutor
     * 然后多个线程同时操作scriptCache，需要保证线程安全。
     *
     * @param expression
     * @return
     */
    private Script getScriptFromCache(String expression) {
        if (scriptCache.containsKey(expression)) {
            return scriptCache.get(expression);
        }
        synchronized (lock) {
            if (scriptCache.containsKey(expression)) {
                return scriptCache.get(expression);
            }
            Script script = groovyShell.parse(expression);
            scriptCache.put(expression, script);
            return script;
        }
    }

    public Object ruleParse(String expression) {
        Script script = getScriptFromCache(expression);
        script.setBinding(groovyBinding);
        return script.run();
    }

    public Object ruleParse(String expression, Map<String, Object> paramMap) {
        Binding binding = groovyBinding;
        paramMap.forEach((key,value)->binding.setProperty(key,value));
        Script script = getScriptFromCache(expression);
        script.setBinding(binding);
        return script.run();
    }




}
