package com.kexun;

import com.alibaba.fastjson.JSONObject;
import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import groovy.lang.Script;
import groovy.util.GroovyScriptEngine;
import org.junit.jupiter.api.Test;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description: Groovy执行
 *
 * @author lgs
 * @date 2023/06/26
 */
public class GroovyTest {


    /**
     * 测试groovy执行
     */
    @Test
    public void test() {
        GroovyShell groovyShell = new GroovyShell();
        //装载解析脚本代码
        Script script = groovyShell.parse("package groovy\n" +
                "\n" +
                "def HelloWorld(){\n" +
                "    println \"hello Groovy\"\n" +
                "}");
        //执行
        script.invokeMethod("HelloWorld", null);

    }

    /**
     * 测试带参数的Groovy脚本
     */
    @Test
    public void test2() {
        //创建GroovyShell
        GroovyShell groovyShell = new GroovyShell();
        //装载解析脚本代码
        Script script = groovyShell.parse("package groovy\n" +
                "\n" +
                "/**\n" +
                " * 简易加法\n" +
                " * @param a 数字a\n" +
                " * @param b 数字b\n" +
                " * @return 和\n" +
                " */\n" +
                "def add(int a, int b) {\n" +
                "    return a + b\n" +
                "}\n" +
                "\n" +
                "/**\n" +
                " * map转化为String\n" +
                " * @param paramMap 参数map\n" +
                " * @return 字符串\n" +
                " */\n" +
                "def mapToString(Map<String, String> paramMap) {\n" +
                "    StringBuilder stringBuilder = new StringBuilder();\n" +
                "    paramMap.forEach({ key, value ->\n" +
                "        stringBuilder.append(\"key:\" + key + \";value:\" + value)\n" +
                "    })\n" +
                "    return stringBuilder.toString()\n" +
                "}");
        //执行加法脚本
        Object[] params1 = new Object[]{1, 2};
        int sum = (int) script.invokeMethod("add", params1);
        System.out.println(sum);
        //执行map转String脚本
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("科目1", "语文");
        paramMap.put("科目2", "数学");
        Object[] params2 = new Object[]{paramMap};
        String result = (String) script.invokeMethod("mapToString", params2);
        System.out.println("mapToString:" + result);

    }

    /**
     * 使用GroovyScriptEngine调用无参
     *
     * @throws Exception
     */
    @Test
    public void test3() throws Exception {
        // GroovyScriptEngine的根路径，如果参数是字符串数组，说明有多个根路径
        GroovyScriptEngine engine = new GroovyScriptEngine("src/main/java/com/kexun/groovy");

        Binding binding = new Binding();
        Object result = engine.run("FunGroove.groovy", binding);
        System.out.println(result);
    }

    /**
     * 使用GroovyScriptEngine调用有参
     * @throws Exception
     */
    @Test
    public void test4() throws Exception{
        GroovyScriptEngine engine = new GroovyScriptEngine("src/main/java/com/kexun/groovy");
        Binding binding = new Binding();

        binding.setVariable("arg","测试参数");

        Object run = engine.run("FunArgGroove.groovy", binding);
        System.out.println(run);
    }


    public static List<JSONObject> invokeMethod(String templateScript, JSONObject configParam) {
        Binding groovyBinding = new Binding();
        GroovyShell groovyShell = new GroovyShell(groovyBinding);
        Script script = groovyShell.parse(templateScript);
        Object[] params = new Object[]{configParam};
        List<JSONObject> resultList = (List<JSONObject>) script.invokeMethod("methodName", params);
        return resultList;
    }

}
